@@ -86,6 +86,8 @@ static void histo_startup() {
 static void histo_close(gboolean revert) {
 	int i;
 	if (revert) {
+		set_cursor_waiting(TRUE);
+
 		for (i = 0; i < gfit.naxes[2]; i++) {
 			set_histogram(hist_backup[i], i);
 			hist_backup[i] = NULL;
@@ -94,6 +96,7 @@ static void histo_close(gboolean revert) {
 		adjust_cutoff_from_updated_gfit();
 		redraw(com.cvport, REMAP_ALL);
 		redraw_previews();
+		set_cursor_waiting(FALSE);
 	}
 
 	// free data
@@ -102,7 +105,7 @@ static void histo_close(gboolean revert) {
 }
 
 static void histo_recompute() {
-	set_cursor_waiting(TRUE);
+	set_cursor("progress");
 	copyfits(&histo_gfit_backup, &gfit, CP_COPYA, -1);
 
 	apply_mtf_to_fits(&histo_gfit_backup, &gfit);
@@ -111,7 +114,6 @@ static void histo_recompute() {
 	adjust_cutoff_from_updated_gfit();
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
-	set_cursor_waiting(FALSE);
 }
 
 static void _init_clipped_pixels() {
@@ -156,31 +158,34 @@ static int is_histogram_visible() {
 
 static void init_toggles() {
 	if (!toggles[0]) {
-		toggles[0] = GTK_TOGGLE_TOOL_BUTTON(
-				gtk_builder_get_object(builder, "histoToolRed"));
-		toggles[1] = GTK_TOGGLE_TOOL_BUTTON(
-				gtk_builder_get_object(builder, "histoToolGreen"));
-		toggles[2] = GTK_TOGGLE_TOOL_BUTTON(
-				gtk_builder_get_object(builder, "histoToolBlue"));
+		toggles[0] = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("histoToolRed"));
+		toggles[1] = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("histoToolGreen"));
+		toggles[2] = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("histoToolBlue"));
 		toggles[3] = NULL;
-		toggleGrid = GTK_TOGGLE_TOOL_BUTTON(
-				gtk_builder_get_object(builder, "histoToolGrid"));
-		toggleCurve = GTK_TOGGLE_TOOL_BUTTON(
-				gtk_builder_get_object(builder, "histoToolCurve"));
+		toggleGrid = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("histoToolGrid"));
+		toggleCurve = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("histoToolCurve"));
 	}
 }
 
 // sets the channel names of the toggle buttons in the histogram window, based on
 // the number of layers of gfit
 static void set_histo_toggles_names() {
+	gchar *image;
+
 	init_toggles();
 
 	if (gfit.naxis == 2) {
-		const char* test = gtk_tool_button_get_label(GTK_TOOL_BUTTON(toggles[0]));
-		if (strcmp(test, "K")) {
-			gtk_tool_button_set_label(GTK_TOOL_BUTTON(toggles[0]), "K");
-			gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), "Gray channel");
+		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Gray channel"));
+		GtkWidget *w;
+		if (com.want_dark) {
+			image = g_build_filename(com.app_path, "pixmaps", "monochrome_dark.png", NULL);
+			w = gtk_image_new_from_file(image);
+		} else {
+			image = g_build_filename(com.app_path, "pixmaps", "monochrome.png", NULL);
+			w = gtk_image_new_from_file(image);
 		}
+		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
+		gtk_widget_show(w);
 		gtk_toggle_tool_button_set_active(toggles[0], TRUE);
 		gtk_widget_set_visible(GTK_WIDGET(toggles[1]), FALSE);
 		gtk_widget_set_visible(GTK_WIDGET(toggles[2]), FALSE);
@@ -193,11 +198,11 @@ static void set_histo_toggles_names() {
 			gtk_widget_set_visible(GTK_WIDGET(toggles[3]), FALSE);
 
 	} else {
-		const char* test = gtk_tool_button_get_label(GTK_TOOL_BUTTON(toggles[0]));
-		if (strcmp(test, "R")) {
-			gtk_tool_button_set_label(GTK_TOOL_BUTTON(toggles[0]), "R");
-			gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), "Red channel");
-		}
+		image = g_build_filename(com.app_path, "pixmaps", "r.png", NULL);
+		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Red channel"));
+		GtkWidget *w = gtk_image_new_from_file(image);
+		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
+		gtk_widget_show(w);
 		gtk_toggle_tool_button_set_active(toggles[0], TRUE);
 		gtk_toggle_tool_button_set_active(toggles[1], TRUE);
 		gtk_toggle_tool_button_set_active(toggles[2], TRUE);
@@ -210,6 +215,7 @@ static void set_histo_toggles_names() {
 			gtk_toggle_tool_button_set_active(toggles[3], TRUE);
 		}
 	}
+	g_free(image);
 }
 
 static double get_histoZoomValueH() {
@@ -418,7 +424,6 @@ static void display_histo(gsl_histogram *histo, cairo_t *cr, int layer, int widt
 
 static void apply_mtf_to_fits(fits *from, fits *to) {
 	int i, chan, nb_chan, ndata;
-	WORD *buf[3] = { from->pdata[RLAYER], from->pdata[GLAYER], from->pdata[BLAYER] };
 	double norm = (double)get_normalized_value(from);
 
 	assert(from->naxes[2] == 1 || from->naxes[2] == 3);
@@ -486,7 +491,7 @@ static void set_ranges_in_the_gui(double min_shadows, double max_highlight) {
 	gtk_range_set_range(scale_transfert_function[2], min_shadows, 1.0);
 }
 
-static void reset_curors_and_values() {
+static void reset_cursors_and_values() {
 	static GtkRange *scale_transfert_function[3] = { NULL, NULL, NULL };
 	if (scale_transfert_function[0] == NULL) {
 		scale_transfert_function[0] = GTK_RANGE(lookup_widget("scale_shadows"));
@@ -724,14 +729,14 @@ void on_histogram_window_show(GtkWidget *object, gpointer user_data) {
 }
 
 void on_button_histo_close_clicked(GtkButton *button, gpointer user_data) {
-	reset_curors_and_values();
+	reset_cursors_and_values();
 	histo_close(TRUE);
 	gtk_widget_hide(lookup_widget("histogram_window"));
 }
 
 void on_button_histo_cancel_clicked(GtkButton *button, gpointer user_data) {
 	set_cursor_waiting(TRUE);
-	reset_curors_and_values();
+	reset_cursors_and_values();
 	histo_close(TRUE);
 	histo_startup();
 	set_cursor_waiting(FALSE);
@@ -751,19 +756,21 @@ void on_button_histo_apply_clicked(GtkButton *button, gpointer user_data) {
 	// partial cleanup
 	fprintf(stdout, "Applying histogram (mid=%.3lf, lo=%.3lf, hi=%.3lf)\n",
 			_midtones, _shadows, _highlights);
-	undo_save_state(&histo_gfit_backup, "Processing: Histogram Transformation "
+	undo_save_state(&histo_gfit_backup, "Processing: Histogram Transf. "
 			"(mid=%.3lf, lo=%.3lf, hi=%.3lf)", _midtones, _shadows, _highlights);
 	clearfits(&histo_gfit_backup);
 	clear_hist_backup();
 	// reinit
 	histo_startup();
-	reset_curors_and_values();
+	reset_cursors_and_values();
+	set_cursor("default");
 }
 
 gboolean on_scale_midtones_button_release_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
 	_midtones = gtk_range_get_value(GTK_RANGE(widget));
 	histo_recompute();
+	set_cursor("grab");
 	return FALSE;
 }
 
@@ -771,6 +778,7 @@ gboolean on_scale_shadows_button_release_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
 	_shadows = gtk_range_get_value(GTK_RANGE(widget));
 	histo_recompute();
+	set_cursor("default");
 	return FALSE;
 }
 
@@ -778,6 +786,7 @@ gboolean on_scale_highlights_button_release_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
 	_highlights = gtk_range_get_value(GTK_RANGE(widget));
 	histo_recompute();
+	set_cursor("default");
 	return FALSE;
 }
 
@@ -854,7 +863,8 @@ void on_histoMidEntry_changed(GtkEditable *editable, gpointer user_data) {
 	g_signal_handlers_block_by_func(MidRange, on_scale_midtones_value_changed, NULL);
 	gtk_range_set_value(MidRange, value);
 	g_signal_handlers_unblock_by_func(MidRange, on_scale_midtones_value_changed, NULL);
-	update_histo_mtf();
+	histo_recompute();
+	set_cursor("default");
 }
 
 void on_histoShadEntry_changed(GtkEditable *editable, gpointer user_data) {
@@ -885,7 +895,8 @@ void on_histoShadEntry_changed(GtkEditable *editable, gpointer user_data) {
 	g_signal_handlers_block_by_func(ShadRange, on_scale_shadows_value_changed, NULL);
 	gtk_range_set_value(ShadRange, _shadows);
 	g_signal_handlers_unblock_by_func(ShadRange, on_scale_shadows_value_changed, NULL);
-	update_histo_mtf();
+	histo_recompute();
+	set_cursor("default");
 }
 
 void on_histoHighEntry_changed(GtkEditable *editable, gpointer user_data) {
@@ -916,7 +927,8 @@ void on_histoHighEntry_changed(GtkEditable *editable, gpointer user_data) {
 	g_signal_handlers_block_by_func(HighRange, on_scale_highlights_value_changed, NULL);
 	gtk_range_set_value(HighRange, _highlights);
 	g_signal_handlers_unblock_by_func(HighRange, on_scale_highlights_value_changed, NULL);
-	update_histo_mtf();
+	histo_recompute();
+	set_cursor("default");
 }
 
 void on_histoZoom100_clicked(GtkButton *button, gpointer user_data) {
@@ -960,21 +972,27 @@ void on_histoToolAutoStretch_clicked(GtkToolButton *button, gpointer user_data)
 gboolean on_scale_midtones_enter_notify_event(GtkWidget *widget,
 		GdkEvent *event, gpointer user_data) {
 
-	set_cursor(GDK_HAND2, TRUE);
+	set_cursor("grab");
+	return FALSE;
+}
+
+gboolean on_scale_midtones_button_press_event(GtkWidget *widget,
+		GdkEvent *event, gpointer user_data) {
+	set_cursor("grabbing");
 	return FALSE;
 }
 
 gboolean on_scale_midtones_leave_notify_event(GtkWidget *widget,
 		GdkEvent *event, gpointer user_data) {
 
-	set_cursor(GDK_HAND2, FALSE);
+	set_cursor("default");
 	return FALSE;
 }
 
 void on_menuitem_histo_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	set_cursor_waiting(TRUE);
 	histo_startup();
-	reset_curors_and_values();
+	reset_cursors_and_values();
 	compute_histo_for_gfit();
 	gtk_widget_show(lookup_widget("histogram_window"));
 	set_cursor_waiting(FALSE);
