@@ -414,7 +414,11 @@ static int exportCSV(pldata *plot, sequence *seq) {
 					if (!seq->imgparam[i].incl)
 						continue;
 					int x = 0;
-					fprintf(csv, "%g", tmp_plot->data[j].x);
+					double date = tmp_plot->data[j].x;
+					if (julian0) {
+						date += julian0;
+					}
+					fprintf(csv, "%.10lf", date);
 					while (x < MAX_SEQPSF && seq->photometry[x]) {
 						fprintf(csv, ", %g", tmp_plot->data[j].y);
 						tmp_plot = tmp_plot->next;
@@ -428,7 +432,11 @@ static int exportCSV(pldata *plot, sequence *seq) {
 				for (i = 0, j = 0; i < plot->nb; i++) {
 					if (!seq->imgparam[i].incl)
 						continue;
-					fprintf(csv, "%g, %g\n", plot->data[j].x, plot->data[j].y);
+					double date = plot->data[j].x;
+					if (julian0) {
+						date += julian0;
+					}
+					fprintf(csv, "%.10lf, %g\n", date, plot->data[j].y);
 					j++;
 				}
 			}
@@ -624,7 +632,7 @@ void on_clearAllPhotometry_clicked(GtkButton *button, gpointer user_data) {
 
 gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 	guint width, height, i, j;
-	double mean;
+	double mean, color;
 	int min, max, nb_graphs = 0;
 	struct kpair *avg;
 	struct kplotcfg cfgplot;
@@ -687,7 +695,9 @@ gboolean on_DrawingPlot_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
 		width = gtk_widget_get_allocated_width(widget);
 		height = gtk_widget_get_allocated_height(widget);
 
-		cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+		color = com.want_dark ? 0.0 : 1.0;
+
+		cairo_set_source_rgb(cr, color, color, color);
 		cairo_rectangle(cr, 0.0, 0.0, width, height);
 		cairo_fill(cr);
 		kplot_draw(p, width, height, cr);
