@@ -28,16 +28,13 @@
 #include "gui/progress_and_log.h"
 #include "gui/callbacks.h"
 #include "gui/save_dialog.h"
+#include "gui/message_dialog.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
 
 
 static image_type whichminisave = TYPEUNDEF;
-#ifdef _WIN32
-static GtkFileChooserNative *saveDialog = NULL;
-#else
-static GtkWidget *saveDialog = NULL;
-#endif
+static SirilWidget *saveDialog = NULL;
 
 static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
 		const gchar *pattern, gboolean set_default) {
@@ -155,13 +152,12 @@ static void set_description_in_TIFF() {
 	gtk_text_buffer_delete(tbuf, &itStart, &itEnd);
 	/* History already written in header */
 	if (gfit.history) {
-		GSList *list = gfit.history;
-		while (list) {
+		GSList *list;
+		for (list = gfit.history; list; list = list->next) {
 			gtk_text_buffer_get_end_iter(tbuf, &itEnd);
 			gtk_text_buffer_insert(tbuf, &itEnd, (gchar *)list->data, -1);
 			gtk_text_buffer_get_end_iter(tbuf, &itEnd);
 			gtk_text_buffer_insert(tbuf, &itEnd, "\n", 1);
-			list = list->next;
 		}
 	}
 	/* New history */
@@ -185,8 +181,7 @@ static void prepare_savepopup(int type) {
 	int tab;
 
 	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(
-				gtk_builder_get_object(builder, "notebookFormat"));
+		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
 		savepopup = lookup_widget("savepopup");
 		savetxt = lookup_widget("filenameframe");
 	}
@@ -233,49 +228,51 @@ static void prepare_savepopup(int type) {
 static void init_dialog() {
 	GtkWindow *parent;
 	if (saveDialog == NULL) {
-		GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SAVE;
 		parent = siril_get_active_window();
-
-#ifdef _WIN32
-		saveDialog = gtk_file_chooser_native_new("Save File", parent, action,
-				_("_Save"), _("_Cancel"));
-#else
-		saveDialog = gtk_file_chooser_dialog_new(_("Save File"), parent, action,
-				_("_Cancel"), GTK_RESPONSE_CANCEL, _("_Save"),
-				GTK_RESPONSE_ACCEPT, NULL);
-#endif				
+		saveDialog = siril_file_chooser_save(parent, GTK_FILE_CHOOSER_ACTION_SAVE);
 	}
 }
 
 static void close_dialog() {
 	if (saveDialog != NULL) {
-#ifdef _WIN32
-		g_object_unref(saveDialog);
-#else
-		gtk_widget_destroy(saveDialog);
-#endif
+		siril_widget_destroy(saveDialog);
 		saveDialog = NULL;
 	}
 }
 
+static gchar *get_filename() {
+	gchar *basename;
+
+	if (sequence_is_loaded() && com.seq.current != RESULT_IMAGE) {
+		char fname[256];
+		/* set the output file name default as the current image.jpg */
+		seq_get_image_filename(&com.seq, com.seq.current, fname);
+		basename = g_path_get_basename(fname);
+	} else {
+		basename = g_path_get_basename(com.uniq->filename);
+	}
+
+	return remove_ext_from_filename(basename);
+}
+
 static int save_dialog() {
 	GtkFileChooser *chooser;
 	GtkFileFilter *filter;
 	GtkEntry *savetext;
 	gint res;
+	gchar *fname;
 
 	init_dialog();
 
 	chooser = GTK_FILE_CHOOSER(saveDialog);
+	fname = get_filename();
 
+	gtk_file_chooser_set_current_name(chooser, fname);
 	gtk_file_chooser_set_do_overwrite_confirmation(chooser, TRUE);
 	set_filters_save_dialog(chooser);
+	g_free(fname);
 
-#ifdef _WIN32
-	res = gtk_native_dialog_run(GTK_NATIVE_DIALOG(saveDialog));
-#else
-	res = gtk_dialog_run(GTK_DIALOG(saveDialog));
-#endif	
+	res = siril_dialog_run(saveDialog);
 	if (res == GTK_RESPONSE_ACCEPT) {
 
 		gchar *filename = gtk_file_chooser_get_filename(chooser);
@@ -297,12 +294,14 @@ static int save_dialog() {
 // idle function executed at the end of the Save Data processing
 gboolean end_save(gpointer p) {
 	struct savedial_data *args = (struct savedial_data *) p;
+	if (args->retval)
+		siril_message_dialog(GTK_MESSAGE_ERROR, _("Error"), _("File saving failed. Check the logs for more info."));
 
 	gtk_entry_set_text(args->entry, "");
 	gtk_widget_hide(lookup_widget("savepopup"));
 	stop_processing_thread();
 	set_cursor_waiting(FALSE);
-	close_dialog();
+	close_dialog();	// is this different from the hide above?
 	update_used_memory();
 	free(args);
 	return FALSE;
@@ -338,26 +337,27 @@ static void initialize_data(gpointer p) {
 static gpointer minisavedial(gpointer p) {
 	struct savedial_data *args = (struct savedial_data *) p;
 	uint32_t bytes_per_sample;
+	args->retval = 0;
 
 	if (args->filename[0] != '\0') {
 		switch (whichminisave) {
 		case TYPEBMP:
-			savebmp(args->filename, &gfit);
+			args->retval = savebmp(args->filename, &gfit);
 			break;
 #ifdef HAVE_LIBJPEG
 		case TYPEJPG:
-			savejpg(args->filename, &gfit, args->quality);
+			args->retval = savejpg(args->filename, &gfit, args->quality);
 			break;
 #endif
 #ifdef HAVE_LIBTIFF
 		case TYPETIFF:
-			savetif(args->filename, &gfit, args->bitspersamples);
+			args->retval = savetif(args->filename, &gfit, args->bitspersamples);
 			break;
 #endif
 #ifdef HAVE_LIBPNG
 		case TYPEPNG:
-			bytes_per_sample = gfit.orig_bitpix > BYTE_IMG ? 2 : 1;
-			savepng(args->filename, &gfit, bytes_per_sample, gfit.naxes[2] == 3);
+			bytes_per_sample = gfit.orig_bitpix != BYTE_IMG ? 2 : 1;
+			args->retval = savepng(args->filename, &gfit, bytes_per_sample, gfit.naxes[2] == 3);
 			break;
 #endif
 		case TYPEFITS:
@@ -381,15 +381,15 @@ static gpointer minisavedial(gpointer p) {
 					gfit.hi = UCHAR_MAX;
 					gfit.lo = 0;
 				}
-				if (gfit.orig_bitpix == BYTE_IMG && gfit.bitpix > BYTE_IMG) {
+				if (gfit.orig_bitpix == BYTE_IMG && gfit.bitpix != BYTE_IMG) {
 					gfit.hi = USHRT_MAX;
 					gfit.lo = 0;
 				}
 			}
-			savefits(args->filename, &gfit);
+			args->retval = savefits(args->filename, &gfit);
 			break;
 		case TYPEPNM:
-			saveNetPBM(args->filename, &gfit);
+			args->retval = saveNetPBM(args->filename, &gfit);
 			break;
 		default:
 			siril_log_message(_("This type of file is not handled. Should not happen"));
@@ -397,7 +397,6 @@ static gpointer minisavedial(gpointer p) {
 		}
 	}
 	siril_add_idle(end_save, args);
-
 	return NULL;
 }
 
@@ -579,24 +578,20 @@ void on_button_cancelpopup_clicked(GtkButton *button, gpointer user_data) {
 }
 
 void on_save1_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkWidget *savepopup = NULL;
+	GtkWidget *savepopup = lookup_widget("savepopup");
 
-	if (savepopup == NULL) {
-		savepopup = lookup_widget("savepopup");
-	}
-
-	int res = save_dialog();
-	if (res == GTK_RESPONSE_ACCEPT) {
+	if (save_dialog() == GTK_RESPONSE_ACCEPT) {
 		/* now it is not needed for some formats */
 		if (whichminisave != TYPEBMP && whichminisave != TYPEPNG
 				&& whichminisave != TYPEPNM) {
+			close_dialog();
 			GtkWindow *parent = siril_get_active_window();
 			if (!GTK_IS_WINDOW(parent)) {
 				parent = GTK_WINDOW(lookup_widget("control_window"));
 			}
 			gtk_window_set_transient_for(GTK_WINDOW(savepopup),	parent);
 			gtk_widget_show(savepopup);
-			close_dialog();
+			gtk_window_present(GTK_WINDOW(savepopup));
 		}
 		else {
 			struct savedial_data *args = malloc(sizeof(struct savedial_data));
