@@ -20,10 +20,10 @@ int cvResizeGaussian_data8(uint8_t *dataIn, int rx, int ry, uint8_t *dataOut,
 		int toX, int toY, int chan, int interpolation);
 int cvTranslateImage(fits *image, point shift, int interpolation);
 int cvRotateImage(fits *, point, double, int, int);
-int cvCalculH(s_star *star_array_img,
+unsigned char *cvCalculH(s_star *star_array_img,
 		struct s_star *star_array_ref, int n, Homography *H);
 int cvApplyScaleToH(Homography *H1, double scale);
-int cvTransformImage(fits *, point, Homography, int);
+int cvTransformImage(fits *image, long width, long height, Homography Hom, int interpolation);
 int cvUnsharpFilter(fits*, double, double);
 int cvComputeFinestScale(fits *image);
 int cvLucyRichardson(fits *image, double sigma, int iterations);
