@@ -76,7 +76,8 @@ static char *tooltip_text[] = { N_("One Star Registration: This is the simplest
 };
 /* callback for the selected area event */
 void _reg_selected_area_callback() {
-	update_reg_interface(TRUE);
+	if (!com.headless)
+		update_reg_interface(TRUE);
 }
 
 static struct registration_method *reg_methods[NUMBER_OF_METHODS];
@@ -112,9 +113,9 @@ void initialize_registration_methods() {
 
 	tip = g_string_new ("");
 	for (j = 0; j < i; j ++) {
-		g_string_append(tip, _(tooltip_text[j]));
+		tip = g_string_append(tip, _(tooltip_text[j]));
 		if (j < i - 1)
-			g_string_append(tip, "\n\n");
+			tip = g_string_append(tip, "\n\n");
 	}
 	ctip = g_string_free (tip, FALSE);
 	gtk_widget_set_tooltip_text(lookup_widget("comboboxregmethod"), ctip);
@@ -214,6 +215,7 @@ int register_shift_dft(struct registration_args *args) {
 			printf("Error allocating registration data\n");
 			return -2;
 		}
+		args->seq->regparam[args->layer] = current_regdata;
 	}
 
 	/* loading reference frame */
@@ -225,6 +227,7 @@ int register_shift_dft(struct registration_args *args) {
 	ret = seq_read_frame_part(args->seq, args->layer, ref_image, &fit_ref,
 			&args->selection, FALSE);
 
+
 	if (ret) {
 		siril_log_message(
 				_("Register: could not load first image to register, aborting.\n"));
@@ -255,8 +258,7 @@ int register_shift_dft(struct registration_args *args) {
 	current_regdata[ref_image].quality = QualityEstimate(&fit_ref, args->layer, QUALTYPE_NORMAL);
 	clearfits(&fit_ref);
 	fftw_execute_dft(p, ref, in); /* repeat as needed */
-	current_regdata[ref_image].shiftx = 0.0;
-	current_regdata[ref_image].shifty = 0.0;
+	set_shifts(args->seq, ref_image, args->layer, 0.0, 0.0, FALSE);
 
 	q_min = q_max = current_regdata[ref_image].quality;
 	q_index = ref_image;
@@ -295,12 +297,9 @@ int register_shift_dft(struct registration_args *args) {
 				for (x = 0; x < sqsize; x++)
 					img[x] = (double) fit.data[x];
 
-				// We don't need fit anymore, we can destroy it.
 				current_regdata[frame].quality = QualityEstimate(&fit, args->layer,
 						QUALTYPE_NORMAL);
 
-				clearfits(&fit);
-
 #ifdef _OPENMP
 #pragma omp critical
 #endif
@@ -340,11 +339,12 @@ int register_shift_dft(struct registration_args *args) {
 					shiftx -= size;
 				}
 
-				/* for Y, it's a bit special because FITS are upside-down */
-				float sign = args->seq->type == SEQ_SER ? -1 : 1;
+				set_shifts(args->seq, frame, args->layer, (float)shiftx, (float)shifty,
+						fit.top_down);
+
+				// We don't need fit anymore, we can destroy it.
+				clearfits(&fit);
 
-				current_regdata[frame].shiftx = (float) shiftx;
-				current_regdata[frame].shifty = (float) shifty * sign;
 
 				/* shiftx and shifty are the x and y values for translation that
 				 * would make this image aligned with the reference image.
@@ -382,7 +382,6 @@ int register_shift_dft(struct registration_args *args) {
 	fftw_free(ref);
 	fftw_free(convol);
 	if (!ret) {
-		args->seq->regparam[args->layer] = current_regdata;
 		if (args->x2upscale)
 			args->seq->upscale_at_stacking = 2.0;
 		else
@@ -391,6 +390,9 @@ int register_shift_dft(struct registration_args *args) {
 		update_used_memory();
 		siril_log_message(_("Registration finished.\n"));
 		siril_log_color_message(_("Best frame: #%d.\n"), "bold", q_index);
+	} else {
+		free(args->seq->regparam[args->layer]);
+		args->seq->regparam[args->layer] = NULL;
 	}
 	return ret;
 }
@@ -401,7 +403,7 @@ int register_shift_dft(struct registration_args *args) {
  */
 int register_shift_fwhm(struct registration_args *args) {
 	int frame, ref_image;
-	float nb_frames, cur_nb;
+	float nb_frames, cur_nb = 0.f;
 	double reference_xpos, reference_ypos;
 	double fwhm_min = DBL_MAX;
 	int fwhm_index = -1;
@@ -441,15 +443,14 @@ int register_shift_fwhm(struct registration_args *args) {
 	fwhm_index = ref_image;
 
 	/* Second step: align image by aligning star coordinates together */
-	for (frame = 0, cur_nb = 0.f; frame < args->seq->number; frame++) {
+	for (frame = 0; frame < args->seq->number; frame++) {
 		double tmp;
 		if (args->run_in_thread && !get_thread_run())
 			break;
 		if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
 			continue;
 		if (frame == ref_image || !current_regdata[frame].fwhm_data) {
-			current_regdata[frame].shiftx = 0.0;
-			current_regdata[frame].shifty = 0.0;
+			set_shifts(args->seq, frame, args->layer, 0.0, 0.0, FALSE);
 			continue;
 		}
 		if (current_regdata[frame].fwhm < fwhm_min
@@ -462,11 +463,6 @@ int register_shift_fwhm(struct registration_args *args) {
 		tmp = current_regdata[frame].fwhm_data->ypos - reference_ypos;
 		current_regdata[frame].shifty = tmp;
 
-		/* shiftx and shifty are the x and y values for translation that
-		 * would make this image aligned with the reference image.
-		 * WARNING: the y value is counted backwards, since the FITS is
-		 * stored down from up.
-		 */
 		fprintf(stderr, "reg: file %d, shiftx=%f shifty=%f\n",
 				args->seq->imgparam[frame].filenum,
 				current_regdata[frame].shiftx, current_regdata[frame].shifty);
@@ -504,6 +500,7 @@ int register_ecc(struct registration_args *args) {
 			printf("Error allocating registration data\n");
 			return -2;
 		}
+		args->seq->regparam[args->layer] = current_regdata;
 	}
 
 	if (args->process_all_frames)
@@ -552,8 +549,7 @@ int register_ecc(struct registration_args *args) {
 			}
 			if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
 				continue;
-			current_regdata[frame].shiftx = 0.0;
-			current_regdata[frame].shifty = 0.0;
+			set_shifts(args->seq, frame, args->layer, 0.0, 0.0, FALSE);
 
 			char tmpmsg[1024], tmpfilename[256];
 
@@ -601,9 +597,8 @@ int register_ecc(struct registration_args *args) {
 						q_min = min(q_min, qual);
 					}
 
-					current_regdata[frame].shiftx = -reg_param.dx;
-					current_regdata[frame].shifty = -reg_param.dy;
-
+					set_shifts(args->seq, frame, args->layer, -reg_param.dx,
+							-reg_param.dy, im.top_down);
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -614,7 +609,7 @@ int register_ecc(struct registration_args *args) {
 			}
 		}
 	}
-	args->seq->regparam[args->layer] = current_regdata;
+
 	if (args->x2upscale)
 		args->seq->upscale_at_stacking = 2.0;
 	else
@@ -624,8 +619,9 @@ int register_ecc(struct registration_args *args) {
 	clearfits(&ref);
 	update_used_memory();
 	siril_log_message(_("Registration finished.\n"));
-	if (failed)
+	if (failed) {
 		siril_log_color_message(_("%d frames were excluded.\n"), "red", failed);
+	}
 	siril_log_color_message(_("Best frame: #%d.\n"), "bold", q_index);
 
 	return 0;
@@ -839,7 +835,8 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	GtkComboBox *cbbt_layers;
 	GtkComboBoxText *ComboBoxRegInter;
 
-	if (get_thread_run()) {
+
+	if (!reserve_thread()) {	// reentrant from here
 		siril_log_message(
 				_("Another task is already in progress, ignoring new request.\n"));
 		return;
@@ -849,10 +846,10 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 		fprintf(stderr, "regparam should have been created before\n");
 		// means that a call to seq_check_basic_data() or
 		// check_or_allocate_regparam() is missing somewhere else
+		unreserve_thread();
 		return;
 	}
 
-	/* getting the selected registration method */
 	method = get_selected_registration_method();
 
 	if (com.selection.w <= 0 && com.selection.h <= 0
@@ -860,9 +857,9 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 		msg = siril_log_message(
 				_("All prerequisites are not filled for registration. Select a rectangle first.\n"));
 		siril_message_dialog( GTK_MESSAGE_WARNING, _("Warning"), msg);
+		unreserve_thread();
 		return;
 	}
-	// TODO: check for reentrance
 
 	reg_args = calloc(1, sizeof(struct registration_args));
 
@@ -887,14 +884,27 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	reg_args->translation_only = gtk_toggle_button_get_active(no_translate);
 	reg_args->x2upscale = gtk_toggle_button_get_active(x2upscale);
 	reg_args->cumul = gtk_toggle_button_get_active(cumul);
-	/* Here we should test available free disk space for Drizzle operation */
-	if (reg_args->x2upscale) {
-		double size = seq_compute_size(reg_args->seq);
-		double diff = test_available_space(size * 4.0); //FIXME: 4 is only ok for x2 Drizzle
-		if (diff < 0.0) {
-			msg = siril_log_message(_("Not enough disk space to perform Drizzle operation!\n"));
-			siril_message_dialog(GTK_MESSAGE_ERROR, _("Not enough disk space"), msg);
+	reg_args->prefix = gtk_entry_get_text(
+			GTK_ENTRY(gtk_builder_get_object(builder, "regseqname_entry")));
+
+	/* We check that available disk space is enough when:
+	 * - activating the subpixel alignment, which requires generating a new
+	 *   sequence with bigger images
+	 * - using global star registration with rotation enabled, also generating a
+	 *   new sequence */
+	if (reg_args->x2upscale ||
+			(method->method_ptr == register_star_alignment &&
+			 !reg_args->translation_only)) {
+		// first, remove the files that we are about to create
+		remove_prefixed_sequence_files(reg_args->seq, reg_args->prefix);
+
+		int nb_frames = reg_args->process_all_frames ? reg_args->seq->number : reg_args->seq->selnum;
+		int64_t size = seq_compute_size(reg_args->seq, nb_frames);
+		if (reg_args->x2upscale)
+			size *= 4;
+		if (test_available_space(size) > 0) {
 			free(reg_args);
+			unreserve_thread();
 			return;
 		}
 	}
@@ -905,8 +915,6 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	reg_args->interpolation = gtk_combo_box_get_active(GTK_COMBO_BOX(ComboBoxRegInter));
 	get_the_registration_area(reg_args, method);	// sets selection
 	reg_args->run_in_thread = TRUE;
-	reg_args->prefix = gtk_entry_get_text(
-			GTK_ENTRY(gtk_builder_get_object(builder, "regseqname_entry")));
 	reg_args->load_new_sequence = FALSE; // only TRUE for global registration. Will be updated in this case
 
 	msg = siril_log_color_message(_("Registration: processing using method: %s\n"),
@@ -915,12 +923,13 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	set_cursor_waiting(TRUE);
 	set_progress_bar_data(msg, PROGRESS_RESET);
 
-	start_in_new_thread(register_thread_func, reg_args);
+	start_in_reserved_thread(register_thread_func, reg_args);
 }
 
 // worker thread function for the registration
 gpointer register_thread_func(gpointer p) {
 	struct registration_args *args = (struct registration_args *) p;
+	int retval;
 
 	args->retval = args->func(args);
 
@@ -930,8 +939,12 @@ gpointer register_thread_func(gpointer p) {
 		args->seq->reference_image = sequence_find_refimage(args->seq);
 	}
 	writeseqfile(args->seq);
-	siril_add_idle(end_register_idle, args);
-	return GINT_TO_POINTER(args->retval);
+	retval = args->retval;
+	if (!siril_add_idle(end_register_idle, args)) {
+		free_sequence(args->seq, TRUE);
+		free(args);
+	}
+	return GINT_TO_POINTER(retval);
 }
 
 // end of registration, GTK thread. Executed when started from the GUI and in
@@ -965,7 +978,6 @@ static gboolean end_register_idle(gpointer p) {
 	gtkosx_application_attention_request(osx_app, INFO_REQUEST);
 	g_object_unref (osx_app);
 #endif
-
 	free(args);
 	return FALSE;
 }
